#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2024 MareroQ

# ---------------------------------------------------------------------

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ------------
#  Change Log 
# ------------
# Rel 1.0: /25-01-2024/ Initial release 
#          
#          Based on distress-selection.scm by Chris Gutteridge
#          Live Preview (gtk2) according to the idea of Tin Tran http://gimpchat.com/viewtopic.php?f=9&t=9586#p284974
# 

from gimpfu import *
import gtk, gimpui
gimpui.gimp_ui_init ()

def newline(x1, y1, x2, y2, x3, y3, x4, y4, x5, y5):
    return [x1,y1]*3+[x2,y2]*3+[x3,y3]*3+[x4,y4]*3+[x5,y5]*3
	
def distress_lp(image, layer, option,select,threshold,spread,granularity,horizontal, vertical,bg, col) :

	for layer in image.layers:
	
		pdb.gimp_image_set_active_layer(image,layer)
		pdb.gimp_item_set_visible(layer, 0)
	active_layer = pdb.gimp_image_get_active_layer(image)
	
	copy = pdb.gimp_layer_new_from_drawable(active_layer, image)
	image.add_layer(copy, 0)
	pdb.gimp_item_set_visible(copy, 1)
	
	pdb.gimp_image_raise_layer_to_top(image, copy)
	pdb.gimp_image_set_active_layer(image,copy)
	
	pdb.gimp_image_set_active_layer(image,copy)

	wl = pdb.gimp_image_width(image)
	hl = pdb.gimp_image_height(image)   
	
	if option==0 or option==1:
		new_vectors_d = pdb.gimp_vectors_new(image,"Border for distress")
		pdb.gimp_image_insert_vectors(image, new_vectors_d, None, -1)
 
	d=select
	if option==0: # "Rectangle"
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_d, 0, 30, newline( d, d, wl-d,d,  wl-d,hl-d, d,hl-d, d,d), 1)
		pdb.gimp_image_select_item(image, 2, new_vectors_d)
		
	if option==1:	# "Ellipse"
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors_d ,wl/2 ,hl/2, wl/2-d ,hl/2-d, 0)	
		pdb.gimp_image_select_item(image, 2, new_vectors_d)

	if option==2:	#"Exising Path"
		active_vectors = pdb.gimp_image_get_active_vectors(image)	
		pdb.gimp_image_set_active_vectors(image, active_vectors)	
		pdb.gimp_image_select_item(image, 2, active_vectors)	
		

	if option==3:	# "Exising Selection"
		non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
		if non_empty == TRUE:
			active_vectors=pdb.plug_in_sel2path(image, copy)
		
		active_vectors = pdb.gimp_image_get_active_vectors(image)	
		pdb.gimp_image_set_active_vectors(image, active_vectors)	
		pdb.gimp_image_select_item(image, 2, active_vectors)
		
	if option==4:	#  "Alpha to Logo"
		pdb.gimp_selection_none(image)	
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,copy)

	if d !=0 and option !=0 or option !=1:		
		pdb.gimp_selection_shrink(image,d)
		
	theLayer = pdb.gimp_layer_new(image, wl, hl, 1, "Distress Selection", 100, 0)
	pdb.gimp_image_insert_layer(image,theLayer,None,-1)
	pdb.gimp_drawable_edit_fill(theLayer, 2)
	
	#pdb.gimp_selection_invert(image) 
	#if option!=3:	
	pdb.gimp_selection_none(image)
	pdb.gimp_layer_scale(theLayer, wl/granularity, hl/granularity, 1)
	pdb.plug_in_spread(image, theLayer, spread, spread)
	pdb.plug_in_gauss_iir2(image, theLayer, horizontal, vertical)
	#pdb.plug_in_gauss_iir(image, theLayer, smooth, inSmoothH, inSmoothV)
	pdb.gimp_layer_scale(theLayer, wl, hl, 1)
	pdb.plug_in_threshold_alpha(image, theLayer, threshold)
	pdb.plug_in_gauss_iir(image, theLayer, 1, 1, 1)
	pdb.gimp_image_select_item(image, 2, theLayer)

	pdb.gimp_image_lower_layer_to_bottom(image, image.layers[0])
	pdb.gimp_image_lower_layer_to_bottom(image, layer)	
	pdb.gimp_selection_invert(image)
	pdb.gimp_drawable_edit_clear(copy)
	pdb.gimp_selection_none(image)
	pdb.gimp_context_push()

	pdb.gimp_context_set_foreground(col)
	if bg==0:	
		pdb.gimp_drawable_edit_fill(theLayer, 0)
	if bg==1:
		pdb.gimp_drawable_edit_fill(theLayer, 3)
	if bg==2:
		pdb.gimp_context_get_pattern()	
		pdb.gimp_drawable_edit_fill(theLayer, 4)
		
	preview_layer=	pdb.gimp_image_merge_down(image, copy, 0)
	preview_layer.name = "Preview Distort"	
	if option==0 or option==1:	
		pdb.gimp_image_remove_vectors(image,image.vectors[0])	
	pdb.gimp_context_pop()
##################################################################################################################################	
# Global variables to store the parameters used for our effect/work to show preview or actual layer when user OK it
global_param = 10
global_param1 = 127 #in this example it's radius
global_param2 = 10   #in this example it's samples
global_param3 = 5
global_param4 = 2
global_param5 = 2
proceed_with_changes = False
proceed_with_changes_x = False
proceed_with_changes_y = False
proceed_with_changes_c = False
proceed_with_changes_d = False
chosen_color = 0
image = 0 #we'll set these when dialog() is called so that we can access them later
drawable = 0
has_preview = False
preview_layer = 0

def apply_effect(layer): #function to do work on either preview layer or actual drawable when user clicks OK
    global image

    if layer.name=="Preview Distort":
        pdb.gimp_image_remove_layer(image,pdb.gimp_image_get_layer_by_name(image,"Preview Distort"))
#"Rectangle","Ellipse","Exising Path","Exising Selection", "Alpha to Logo"		
    if selected_option== "Rectangle" :
        option=0	
    if selected_option== "Ellipse" :
        option=1
    if selected_option== "Exising Path" :
        option=2
    if selected_option== "Exising Selection" :
        option=3
    if selected_option== "Alpha to Logo" :
        option=4

    if selected_option_2== "Color" :
        bg=0	
    if selected_option_2== "Transparent" :
        bg=1
    if selected_option_2== "Active Pattern" :
        bg=2	
    #distress(image, layer, option,select,threshold,spread,granularity,smooth,(0,0,0),inSmoothH, inSmoothV)
    distress_lp(image, layer, option,global_param,global_param1,global_param2,global_param3,global_param4,global_param5,bg,chosen_color)
    gimp.displays_flush()

					
def apply_final(layer): #wrapper to apply effect on final and remove preview_layer meant to be called by on_ok_button_clicked

    global preview_layer


    gimp.displays_flush()


# Function to update the live preview
def update_live_preview(): #this is called everytime some parameter changes
    global global_param, global_param1, global_param2, global_param3, global_param4, global_param5
    global image,drawable, vector
    global has_preview,preview_layer #deal with preview layer
   
    # Apply your plugin's effect using the current parameters
    # Use global_param1 and global_param2 to access the user's inputs

        #pdb.gimp_image_remove_layer(image,image.layers[0])
    #debug message
    #pdb.gimp_message(str(global_param1)+","+str(global_param2)+","+str(global_param3)+","+str(global_param4))

    if proceed_with_changes==1:

        apply_effect(image.layers[0])
        
    if proceed_with_changes==0 and proceed_with_changes_d==1:
        pdb.gimp_image_remove_layer(image,image.layers[0])
        #pass       	
    
   
		
            #image.vectors[0].visible = True		

def dialog(image_, drawable_):
    global image, drawable
    if chosen_color==0:
        global chosen_color
        chosen_color = (255, 255, 255)  # Set default color
    image = image_ #for speed and also when user undo it doesn't see our preview creations/deletions
    drawable = drawable_ 

    dialog = gtk.Dialog("MrQ Distress Live Preview", None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
    dialog.set_default_size(300, 100)
	
    #-------------------------------------------------------------
	
    hbox0 = gtk.HBox()
    dialog.vbox.pack_start(hbox0, expand=True, fill=True)
    pattern_label = gtk.Label("\n    Extended distress-selection.scm functions with Live Preview.\nIt is recommended to change the parameters when Preview = No.\n")	
    hbox0.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
	
    # Combo box
    hbox7 = gtk.HBox()
    dialog.vbox.pack_start(hbox7, expand=True, fill=True)
    label7 = gtk.Label("Area:             ")
    hbox7.pack_start(label7, expand=False, fill=False, padding=5)   

    combo_box = gtk.ComboBox()
    hbox7.pack_start(combo_box, expand=False, fill=False, padding=5)

    # Create a ListStore model for ComboBox
    list_store = gtk.ListStore(str)
    options = ["Rectangle","Ellipse","Exising Path","Exising Selection", "Alpha to Logo"]  # Add your options here
    for option in options:
        list_store.append([option])

    # Set the model for ComboBox
    combo_box.set_model(list_store)

    # Create a CellRendererText to render the text in the ComboBox
    cell_renderer = gtk.CellRendererText()

    # Pack the CellRendererText into the ComboBox
    combo_box.pack_start(cell_renderer, True)
    combo_box.add_attribute(cell_renderer, 'text', 0)
  
    combo_box.connect("changed", on_combobox_changed)

    default_option = "Rectangle"
    default_iter = list_store.get_iter_first()
    while default_iter is not None:
        if list_store.get_value(default_iter, 0) == default_option:
            combo_box.set_active_iter(default_iter)
            break
        default_iter = list_store.iter_next(default_iter)
    #-------------------------------------------------------------

    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    # -------------------------------------------------------------
	
    hbox = gtk.HBox()
    dialog.vbox.pack_start(hbox, expand=True, fill=True)
    label = gtk.Label("Reduce area\n(shrink):")
    hbox.pack_start(label, expand=False, fill=False, padding=5)
    adjustment = gtk.Adjustment(value=10, lower=0, upper=1000, step_incr=1, page_incr=0)
    param_scale = gtk.HScale(adjustment)
    param_scale.set_digits(0)  # Display only integers
    hbox.pack_start(param_scale, expand=True, fill=True, padding=5)
    label = gtk.SpinButton(adjustment,0, 0 )
    hbox.pack_start(label, expand=False, fill=False, padding=5)	
    param_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param_scale.connect("value-changed", on_param_changed)

    # -------------------------------------------------------------
	
    hbox1 = gtk.HBox()
    dialog.vbox.pack_start(hbox1, expand=True, fill=True)
    label1 = gtk.Label("\nThreshold:\n( <- bigger\n -> smaller)")
    hbox1.pack_start(label1, expand=False, fill=False, padding=5)
    adjustment1 = gtk.Adjustment(value=127, lower=1, upper=254, step_incr=1, page_incr=0)
    param1_scale = gtk.HScale(adjustment=adjustment1)
    param1_scale.set_digits(0)  # Display only integers
    hbox1.pack_start(param1_scale, expand=True, fill=True, padding=5)
    label1 = gtk.SpinButton(adjustment1,0, 0 )
    hbox1.pack_start(label1, expand=False, fill=False, padding=5)
    param1_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param1_scale.connect("value-changed", on_param1_changed)

    #-------------------------------------------------------------
	
    hbox2 = gtk.HBox()
    dialog.vbox.pack_start(hbox2, expand=True, fill=True)
    label2 = gtk.Label("\nSpread:       ")
    hbox2.pack_start(label2, expand=False, fill=False, padding=5)
    adjustment2 = gtk.Adjustment(value=10, lower=0, upper=200, step_incr=1, page_incr=0)
    param2_scale = gtk.HScale(adjustment=adjustment2)
    param2_scale.set_digits(0)  # Display only integers
    hbox2.pack_start(param2_scale, expand=True, fill=True, padding=5)
    label2 = gtk.SpinButton(adjustment2,0, 0 )
    hbox2.pack_start(label2, expand=False, fill=False, padding=5)
    param2_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param2_scale.connect("value-changed", on_param2_changed)

    #-------------------------------------------------------------
	
    hbox3 = gtk.HBox()
    dialog.vbox.pack_start(hbox3, expand=True, fill=True)
    label3 = gtk.Label("\nGranularity:\n(1 is low)")
    hbox3.pack_start(label3, expand=False, fill=False, padding=5)
    adjustment3 = gtk.Adjustment(value=5, lower=1, upper=20, step_incr=1, page_incr=0)
    param3_scale = gtk.HScale(adjustment=adjustment3)
    param3_scale.set_digits(0)  # Display only integers
    hbox3.pack_start(param3_scale, expand=True, fill=True, padding=5)
    label3 = gtk.SpinButton(adjustment3,0, 0 )
    hbox3.pack_start(label3, expand=False, fill=False, padding=5)
    param3_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param3_scale.connect("value-changed", on_param3_changed)

    #-------------------------------------------------------------
	
    hbox4 = gtk.HBox()
    dialog.vbox.pack_start(hbox4, expand=True, fill=True)
    label4 = gtk.Label("\nSmooth\nHorizontally:")
    hbox4.pack_start(label4, expand=False, fill=False, padding=5)
    adjustment4 = gtk.Adjustment(value=2, lower=0, upper=150, step_incr=1, page_incr=0)
    param4_scale = gtk.HScale(adjustment=adjustment4)
    param4_scale.set_digits(0)  # Display only integers
    hbox4.pack_start(param4_scale, expand=True, fill=True, padding=5)
    label4 = gtk.SpinButton(adjustment4,0, 0 )
    hbox4.pack_start(label4, expand=False, fill=False, padding=5)
    param4_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param4_scale.connect("value-changed", on_param4_changed)
	
    #-------------------------------------------------------------
	
    hbox5 = gtk.HBox()
    dialog.vbox.pack_start(hbox5, expand=True, fill=True)
    label5 = gtk.Label("\nSmooth\nVertically:    ")
    hbox5.pack_start(label5, expand=False, fill=False, padding=5)
    adjustment5 = gtk.Adjustment(value=2, lower=0, upper=150, step_incr=1, page_incr=0)
    param5_scale = gtk.HScale(adjustment=adjustment5)
    param5_scale.set_digits(0)  # Display only integers
    hbox5.pack_start(param5_scale, expand=True, fill=True, padding=5)
    label5 = gtk.SpinButton(adjustment5,0, 0 )
    hbox5.pack_start(label5, expand=False, fill=False, padding=5)
    param5_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS) ######		
    param5_scale.connect("value-changed", on_param5_changed)	
	
    #-------------------------------------------------------------
	
    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)	
 
    #-------------------------------------------------------------
	
    hbox40 = gtk.HBox()
    dialog.vbox.pack_start(hbox40, expand=True, fill=True)
    pattern_label = gtk.Label("")	
    hbox40.pack_start(pattern_label, expand=True, fill=True, padding=1)	
	
    #-------------------------------------------------------------
	
    hbox12 = gtk.HBox()
    dialog.vbox.pack_start(hbox12, expand=True, fill=True)
    toggle_button = gtk.ToggleButton()
    dialog.action_area.add(toggle_button)	
    update_toggle_button_label(toggle_button)
    toggle_button.connect("toggled", on_toggle_button_toggled)
    hbox12.pack_start(toggle_button, expand=True, fill=True, padding=5) 
	
    #-------------------------------------------------------------
	
    hbox0 = gtk.HBox()
    dialog.vbox.pack_start(hbox0, expand=True, fill=True)
    pattern_label = gtk.Label("                       ")
    dialog.action_area.add(pattern_label)	
    hbox0.pack_start(pattern_label, expand=True, fill=True, padding=1)
	
    #-------------------------------------------------------------
	
    # Combo box
    hbox7 = gtk.HBox()
    dialog.vbox.pack_start(hbox7, expand=True, fill=True)
    label7 = gtk.Label("Background:")
    hbox7.pack_start(label7, expand=False, fill=False, padding=5)   

    # Create a ComboBox
    combo_box = gtk.ComboBox()
    hbox7.pack_start(combo_box, expand=False, fill=False, padding=5)

    # Create a ListStore model for ComboBox
    list_store = gtk.ListStore(str)
    options = ["Color", "Transparent", "Active Pattern"] # Add your options here
    for option in options:
        list_store.append([option])

    # Set the model for ComboBox
    combo_box.set_model(list_store)

    # Create a CellRendererText to render the text in the ComboBox
    cell_renderer = gtk.CellRendererText()

    # Pack the CellRendererText into the ComboBox
    combo_box.pack_start(cell_renderer, True)
    combo_box.add_attribute(cell_renderer, 'text', 0)

    # Set up the "changed" signal handler   
    combo_box.connect("changed", on_combobox_changed_2)

    # Set the default option
    default_option = "Color"
    default_iter = list_store.get_iter_first()
    while default_iter is not None:
        if list_store.get_value(default_iter, 0) == default_option:
            combo_box.set_active_iter(default_iter)
            break
        default_iter = list_store.iter_next(default_iter)
		
    #-------------------------------------------------------------
		
    # Create a label for the color picker
    label5 = gtk.Label("\n                  color: ")
    hbox7.pack_start(label5, expand=False, fill=False, padding=5)

    # Create a color picker button
    color_button = gtk.ColorButton()
    color_button.set_use_alpha(False)  # Set to True if you want to include alpha channel
    color_button.set_color(gtk.gdk.Color(chosen_color[0]*256, chosen_color[1]*256, chosen_color[2]*256))
    color_button.connect("color-set", on_color_changed)
    hbox7.pack_start(color_button, expand=False, fill=False, padding=5)	
	
    # -------------------------------------------------------------
	
    hbox20 = gtk.HBox()
    dialog.vbox.pack_start(hbox20, expand=False, fill=False)
    pattern_label = gtk.Label("\n       Preview:")
    #dialog.action_area.add(pattern_label)	
    hbox20.pack_start(pattern_label, expand=False, fill=False, padding=1)
	
    # -------------------------------------------------------------
	
    # Add an CANCEL button
    cancel_buton = dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
    cancel_buton.connect("clicked", cancel) 
	
    # -------------------------------------------------------------

    # Add an OK button
    ok_button = dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
    ok_button.connect("clicked", on_ok_button_clicked)
    # -------------------------------------------------------------	
	
    # Show the dialog
    dialog.show_all()
    dialog.run()
	
#######################################################
	
def on_combobox_changed(combobox):
    global selected_option
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option = model.get_value(active_iter, 0)
    update_live_preview()
	
def on_combobox_changed_2(combobox):
    global selected_option_2
    model = combobox.get_model()
    active_iter = combobox.get_active_iter()

    if active_iter:
        selected_option_2 = model.get_value(active_iter, 0)
    update_live_preview()
	
def on_color_changed(widget, data=None):
    global chosen_color
    color = widget.get_color()
    chosen_color = (int(color.red/256), int(color.green/256), int(color.blue/256))
    update_live_preview()
    pdb.gimp_displays_flush()
	
def on_param_changed(scale):
    global global_param
    global_param = scale.get_value()
    update_live_preview()

def on_param1_changed(scale):
    global global_param1
    global_param1 = scale.get_value()
    update_live_preview()

def on_param2_changed(scale):
    global global_param2
    global_param2 = scale.get_value()
    update_live_preview()
	
def on_param3_changed(scale):
    global global_param3
    global_param3 = scale.get_value()
    update_live_preview()
	
def on_param4_changed(scale):
    global global_param4
    global_param4 = scale.get_value()
    update_live_preview()
	
def on_param5_changed(scale):
    global global_param5
    global_param5 = scale.get_value()
    update_live_preview()


# ------------------------------------------------------------	
def on_toggle_button_toggled(button):
    global proceed_with_changes
    proceed_with_changes = not proceed_with_changes
    update_toggle_button_label(button)

def update_toggle_button_label(button):
    label = "Click to Stop " if proceed_with_changes else "Click to Start"
    button.set_label(label)	
    update_live_preview()


# ------------------------------------------------------------	

def cancel(cancel_buton, data=None):
    global layer
    pdb.gimp_image_remove_layer(image,pdb.gimp_image_get_layer_by_name(image,"Preview Distort"))
	
def on_ok_button_clicked(button, data=None):
    global drawable
    #apply_final(drawable)
    button.get_toplevel().destroy() #destroys the gtk dialog window

register(
    "python_fu_distor_lp",
    "Distort the area",
    "Create a live preview of an effect",
    "MrQ",
    "GPL",
    "2024",
    "<Image>/Select/Modify/Distress Live Preview...",  # Menu location
    "*",  # Image type
    [],
    [],
    dialog
)

main()